
class 'Expander'

function Expander:__init()

  local tool = renoise.tool()

  self.timer_func = function()
    local last_seq_idx = #song.sequencer.pattern_sequence
    local selected_seq_idx = song.selected_sequence_index
    if last_seq_idx == selected_seq_idx and last_seq_idx < 1000 then
      song.sequencer:insert_new_pattern_at(last_seq_idx+1)
    elseif last_seq_idx > selected_seq_idx then
      for seq = last_seq_idx, selected_seq_idx+2,-1 do
        if self:sequence_is_void(seq) then
          local pattern = song:pattern(song.sequencer.pattern_sequence[seq])
          song.sequencer:delete_sequence_at(seq)
        else
          return
        end
      end
    end
    tool:remove_timer(self.timer_func)
  end
  
  self.sequence_observable_func = function()
    if not tool:has_timer(self.timer_func) then
      tool:add_timer(self.timer_func, 10)
    end
  end
  
end

function Expander:sequence_is_void(seq_index)
  local pattern_index = song.sequencer.pattern_sequence[seq_index]
  if song:pattern(pattern_index).is_really_empty and
    not song.sequencer:sequence_is_start_of_section(seq_index) then
    return true
  end
end

function Expander:start()
  self.sequence_observable_func() 
  local observable = song.selected_sequence_index_observable
  if not observable:has_notifier(self.sequence_observable_func) then
    observable:add_notifier(self.sequence_observable_func)
  end
end

function Expander:stop()
  local observable = song.selected_sequence_index_observable
  if observable:has_notifier(self.sequence_observable_func) then
    observable:remove_notifier(self.sequence_observable_func)
  end
end

-- improvement of native renoise.Pattern class

renoise.Pattern.is_really_empty = property(
  function(obj)
    if obj.is_empty then
      for _, patterntrack in ipairs(obj.tracks) do
        if #patterntrack.automation > 0 then
          return false
        end
      end
    return true
    end
  return false
  end
)

-- prefs, start-up, menu

song, app = nil

prefs = renoise.Document.create("ScriptingToolPreferences") {
  auto_grow_sequence = false
}

renoise.tool().preferences = prefs

prefs.auto_grow_sequence:add_notifier(function()
  if prefs.auto_grow_sequence.value then
    app = app or Expander()
    app:start()
  elseif app then
    app:stop()
  end
end)

function song_update()
  song = renoise.song()
  if prefs.auto_grow_sequence.value then
    app = app or Expander()
    app:start()
  end  
end

renoise.tool().app_new_document_observable:add_notifier(song_update)
pcall(song_update)

renoise.tool():add_menu_entry {
  name = "Pattern Sequencer:Auto Grow Sequence",
  invoke = function()
    prefs.auto_grow_sequence.value = not prefs.auto_grow_sequence.value
  end,
  selected = function() return prefs.auto_grow_sequence.value end
}

